clc;clear all;close all;

%% GFSK Baseband Signal's parameters 
symbolrate=50e3; %GFSK's symbolrate
freq_dev=25e3;   %GFSK's frequency deviation
fm=freq_dev/symbolrate; %GFSK's modulation index


%% Gaussian filter 
% Define filter-related parameters.
bt =0.5;         % 3-dB bandwidth-symbol time
of =64;          % Oversampling factor
nt =3 ;          % 3 symbol periods to the filters peak

h = gaussfir(bt,nt,of); 

%% Transimitted data
load data.m;                   % import baseband symbol
N=length(data);                % the number of GFSK symbol
% N=1000;  % the number of GFSK symbol
% data=randi([0,1],[N,1]); % Generate the random symbols

%% Oversample and Gaussian filter

b=data*2-1;
b(N,1)=b(1,1);

len=length(b);
Pt=ones(of,1);

bb=kron(b,Pt);

M=N*of; 
% rr_g=conv(h,bb);
% syncpoint=nt*of;
% rr_g=rr_g(syncpoint+1:length(rr_g)-syncpoint);
rr_g=ifft(fft(h.',M).*fft(bb,M));

%% GFSK modulation
rr_m=zeros(1,len*of);
rr_m(1)=rr_g(1);
for i=2:len*of
    rr_m(i)=rr_m(i-1)+rr_g(i)/of;
end
fai=2*pi*fm*rr_m;

I=cos(fai);
Q=sin(fai);

%generate .csv format waveform file
Z = [I',Q'];
Z=single(Z);
charSymbolrate=num2str(symbolrate/1000);
charFreq_DEV=num2str(freq_dev/1000);
csvName = ['GFSK_SymbolRate',charSymbolrate,'kbps_FreqDev',charFreq_DEV, 'kHz.csv'];
csvwrite(csvName,Z,0,0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
charFs=num2str(symbolrate*of);  % Arb Sample Rate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Send IQ data to arb

vAddress = ['USB0::0x0957::0x2C07::MY52800259::0::INSTR']; %build visa address string to connect
fgen = visa('AGILENT',vAddress); %build IO object
fgen.Timeout = 1000; %set IO time out
%calculate output buffer size
buffer = length(I)*8;
set (fgen,'OutputBufferSize',(buffer+125));

%open connection to 33522
try
   fopen(fgen);
catch exception %problem occurred throw error message
    uiwait(msgbox('Error occurred trying to connect to the 33522, verify correct IP address','Error Message','error'));
    rethrow(exception);
    fclose(fgen);
end

%Query Idendity string and report
fprintf (fgen, '*IDN?');
idn = fscanf (fgen);
fprintf (idn)
fprintf ('\n\n')

mes = ['Connected to ' idn ' sending waveforms.....'];
h = waitbar(0,mes);

%Reset instrument
fprintf (fgen, '*RST');

%turn it to column vector
IQ = [I Q];
IQ = single(IQ);

%Clear volatile memory
fprintf(fgen, 'SOURce1:DATA:VOLatile:CLEar');
fprintf(fgen, 'SOURce2:DATA:VOLatile:CLEar');
fprintf(fgen, 'FORM:BORD SWAP');  %configure the box to correctly accept the binary arb points
fprintf(fgen,'DATA:ARB2:FORMAT AABB'); %set IQ arb format
%update waitbar
waitbar(.1,h,mes);
%send I data to 33522 and setup channel 1
iBytes=num2str(length(IQ) * 4); %# of bytes
header= ['SOURce1:DATA:ARB2 IQDATA, #' num2str(length(iBytes)) iBytes]; %create header
%header= ['SOURce1:DATA:ARBitrary IDATA, #' num2str(length(iBytes)) iBytes]; %create header
binblockBytes = typecast(IQ, 'uint8');  %convert datapoints to binary before sending
fwrite(fgen, [header binblockBytes], 'uint8'); %combine header and datapoints then send to instrument
fprintf(fgen, '*WAI');   %Make sure no other commands are exectued until arb is done downloadin
%Set desired configuration for channel 1
%update waitbar
waitbar(.4,h,mes);


fprintf(fgen,'SOURce1:FUNCtion:ARBitrary IQDATA'); % set current arb waveform to defined arb testrise
fprintf(fgen,'MMEM:STOR:DATA1 "INT:\IQDATA.arb"');%store arb in intermal NV memory
%fprintf(fgen,'SOURCE1:VOLT 2.0'); % set max waveform amplitude to 2 Vpp
fprintf(fgen,'SOURCE1:VOLT:OFFSET 0'); % set offset to 0 V
fprintf(fgen,'OUTPUT1:LOAD 50'); % set output load to 50 ohms
c1SampleRate=['SOURCE1:FUNCtion:ARB:SRATe  ' charFs];
fprintf(fgen,c1SampleRate); % set Arb sample rate
fprintf(fgen,'SOURce1:FUNCtion ARB'); % turn on arb function
fprintf('"IQ" signal downloaded\n\n');
%update waitbar
waitbar(.55,h,mes);

%set the amplitude for each channel
fprintf(fgen,'SOURCE1:VOLT 1'); % set max waveform amplitude to 2 Vpp
fprintf(fgen,'SOURCE2:VOLT 1'); % set max waveform amplitude to 2 Vpp
%Phase sync both channels
fprintf(fgen,'PHAS:SYNC');
%Set Filter off
%fprintf(fgen,'SOURce1:FUNCtion:ARBitrary:FILTer OFF'); % set current arb waveform to defined arb testrise
%fprintf(fgen,'SOURce2:FUNCtion:ARBitrary:FILTer OFF'); % set current arb waveform to defined arb testrise
fprintf(fgen,'FUNC:ARB:SYNC');

%Enable Output for channel 1
fprintf(fgen,'OUTPUT1 ON');

%get rid of message box
waitbar(1,h,mes);
delete(h);

%Read Error
fprintf(fgen, 'SYST:ERR?');
errorstr = fscanf (fgen);

% error checking
if strncmp (errorstr, '+0,"No error"',13)
   errorcheck = 'Arbitrary waveform generated without any error\n';
   fprintf (errorcheck)
else
   errorcheck = ['Error reported: ', errorstr];
   fprintf (errorcheck)
end

fclose(fgen);

